#' @importClassesFrom Biobase ExpressionSet

#' @title mde
#' Generic for testing marginal differential expression
#' @param x ExpressionSet
#' @param y Experimental factor 
#' @param test Hypothesis testing method
#' @param correction Correction method for multiple comparisons
#' @param ... Additional parameters
#' @description 
#' Generic for testing marginal differential expression
#' @export
setGeneric("mde", function(x,y,test,correction,...) 
  standardGeneric("mde"))

#' @title mde
#' Method `mde` for ExpressionSet
#' @param x ExpressionsSet
#' @param y Experimental factor 
#' @param test Statistical procedure (c("rowtt","rowttmod"))
#' @param correction Correction method for multiple comparisons
#' @param ... Additional parameters
#' @description 
#' Method `mde` for ExpressionSet
#' @export
setMethod(f = "mde",
          signature = c(x="ExpressionSet",y="factor",
                        test="list",correction ="character"),
          definition = function(x,y,test,correction,...){
            rowtest(x=Biobase::exprs(x),
                    y=y,
                    test=test[[1]],
                    correction = correction)})


#' @title plot
#' `plot` method for `ExpressionSet`.
#' @description 
#' Some `plot`s for `ExpressionSet`
#' @param x `ExpressionSet`
#' @param y Type of plot: `1` plots the means at x axis and the standard
#' deviations at y axis. `2` plots the medians at x axis and the interquantile
#' ranges at y axis.
#' @param ... Additional parameters
#' @export 
#' 
setMethod(f = "plot",
          signature = c(x="ExpressionSet",y="numeric"),
          definition = function(x,y,...){
            if(y == 1){
              mu0 = rowMeans(Biobase::exprs(x))
              sd0 = matrixStats::rowSds(Biobase::exprs(x))
              df = data.frame(mu0,sd0)
              df = df[sort(mu0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=mu0,y=sd0)) + 
                       ggplot2::geom_smooth() + 
                       ggplot2::xlab("Mean") + 
                       ggplot2::ylab("Standard deviation"))
            }
            if(y == 2){
              median0 = matrixStats::rowMedians(Biobase::exprs(x))
              iqr0 = matrixStats::rowIQRs(Biobase::exprs(x))
              df = data.frame(median0,iqr0)
              df = df[sort(median0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=median0,y=iqr0)) + 
                       ggplot2::geom_smooth() + ggplot2::xlab("Median") + 
                       ggplot2::ylab("Interquantile Range"))
            }
            if(y == 3){
              xt = t(Biobase::exprs(x))
              df = data.frame(xt,groups = Biobase::pData(x)[,"groups"])
              ggplot2::autoplot(stats::prcomp(xt),data=df,colour="groups")
            }
            
            
          })
