#' @importClassesFrom SummarizedExperiment RangedSummarizedExperiment

#' @title mde
#' Method `mde` for `RangedSummarizedExperiment`
#' @param x `RangedSummarizedExperiment`
#' @param y Experimental factor 
#' @param test Statistical procedure (c("edgeRcommon","edgeRtagwise"))
#' @param correction Correction method for multiple comparisons
#' @param ... Additional parameters
#' @description 
#' Method `mde` for `RangedSummarizedExperiment`
#' @export
setMethod(f = "mde",
          signature = c(x="RangedSummarizedExperiment",y="factor",
                        test="list",correction ="character"),
          definition = function(x,y,test,correction,...){
            rowtest(x=SummarizedExperiment::assay(x),
                    y=y,
                    test=test[[1]],
                    correction = correction)})

#' @title plot
#' `plot` method for `RangedSummarizedExperiment`.
#' @description 
#' Some `plot`s for `RangedSummarizedExperiment`
#' @param x `RangedSummarizedExperiment`
#' @param y Type of plot: `1` plots the means at x axis and the standard
#' deviations at y axis. `2` plots the medians at x axis and the interquantile
#' ranges at y axis.
#' @param ... Additional parameters
#' @export 
#' 
setMethod(f = "plot",
          signature = c(x="RangedSummarizedExperiment",y="numeric"),
          definition = function(x,y,...){
            if(y == 1){
              mu0 = rowMeans(SummarizedExperiment::assay(x))
              sd0 = matrixStats::rowSds(SummarizedExperiment::assay(x))
              df = data.frame(mu0,sd0)
              df = df[sort(mu0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=mu0,y=sd0)) + 
                       ggplot2::geom_smooth() + 
                       ggplot2::xlab("Mean") + ggplot2::ylab("Standard deviation"))
            }
            if(y == 2){
              median0 = matrixStats::rowMedians(SummarizedExperiment::assay(x))
              iqr0 = matrixStats::rowIQRs(SummarizedExperiment::assay(x))
              df = data.frame(median0,iqr0)
              df = df[sort(median0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=median0,y=iqr0)) + 
                       ggplot2::geom_smooth() + ggplot2::xlab("Median") + 
                       ggplot2::ylab("Interquantile Range"))
            }
            
          })
