##########################################################
################## Generic functions  ####################
##########################################################

#' Generic function for \code{exprm}
#' @description 
#' Accesor for \code{exprm}
#' @param object \code{ExpressionData} 
#' @export 
#' 
setGeneric("exprm", function(object) standardGeneric("exprm"))

#' Generic function for \code{exprm<-}
#' @description 
#' Generic function for \code{exprm<-}
#' @param object \code{exprm} slot of \code{ExpressionData}
#' @param value for \code{exprm} slot of \code{ExpressionData}
#' @export
#' 
setGeneric("exprm<-",
           function(object, value) standardGeneric("exprm<-"))

#' Generic function for \code{groups} 
#' @description 
#' Generic function for \code{groups} 
#' @param object \code{ExpressionData} 
#' @export
setGeneric("groups", function(object) standardGeneric("groups"))

#' Generic function for  \code{groups<-}
#' @description 
#' Accesor for \code{groups}
#' @param object groups slot of ExpressionData
#' @param value for groups slot of ExpressionData
#' @export
#' 
setGeneric("groups<-",
           function(object, value) standardGeneric("groups<-"))

#' Generic function for \code{type}
#' Accesor for  \code{type}
#' @description 
#' Accesor for \code{type}
#' @param object type of dataset  
#' @export
setGeneric("type", function(object) standardGeneric("type"))

#' Generic function for \code{type<-}
#' @description 
#' Accesor for \code{type<-}
#' @param object type slot of \code{ExpressionData}
#' @param value for type slot of \code{ExpressionData}
#' @export
#' 
setGeneric("type<-",
           function(object, value) standardGeneric("type<-"))

#' Generic function for \code{GeneData} 
#' @description 
#' Accesor for \code{GeneData} 
#' @param object \code{GeneData}  
#' @export
setGeneric("GeneData", function(object) standardGeneric("GeneData"))

#' Generic function \code{GeneData<-}
#' @description 
#' Accesor for \code{GeneData}
#' @param object \code{DifferentialExpressionOutput}
#' @param value for \code{GeneData} slot of \code{DifferentialExpressionOutput}
#' @export
#' 
setGeneric("GeneData<-",
           function(object, value) standardGeneric("GeneData<-"))

#' Generic function for \code{GeneStat} 
#' @description 
#' Generic function for \code{GeneStat}
#' @param object   \code{DifferentialExpressionOutput}
#' @export
setGeneric("GeneStat", function(object) standardGeneric("GeneStat"))

#' Generic function \code{GeneStat<-}
#' @description 
#' Accesor for \code{GeneStat}
#' @param object GeneStat slot 
#' @param value for \code{GeneStat} slot 
#' @export
#' 
setGeneric("GeneStat<-",
           function(object, value) standardGeneric("GeneStat<-"))

#' @title test
#' @param object \code{DifferentialExpressionInput}
#' @description 
#' Accesor for \code{test}, statistical test used to test the 
#' differential expression.
#' @export
setGeneric("test", function(object) standardGeneric("test"))

#' @title correction
#' @description 
#' Accesor for \code{correction} 
#' @param object \code{DifferentialExpressionInput}
#' @export
#' 
setGeneric("correction", function(object) standardGeneric("correction"))

#' Generic function
#' @description 
#' Generic function for \code{GeneSetList} 
#' @param object \code{GSAInput}, \code{GSAOutput}
#' @export
#' 
setGeneric("GeneSetList", function(object) standardGeneric("GeneSetList"))

#' Generic function 
#' @description 
#' Generic function for \code{GeneSetList<-} 
#' @param object \code{GSAInput}, \code{GSAOutput}
#' @param value for \code{GeneSetList}
#' @export
#' 
setGeneric("GeneSetList<-", function(object,value) 
  standardGeneric("GeneSetList<-"))

#' Generic function \code{glimpse}
#' @description 
#' Glimpse statistical output mainly for html report generation  
#' @param  object To be glimpse 
#' @export
setGeneric("glimpse",def=function(object){standardGeneric("glimpse")})


#' Generic function \code{tidy}
#' @description 
#' Tidy statistical output mainly for \code{data.frame} generation 
#' @param  object To be tidy
#' @export
#'  
setGeneric("tidy",def=function(object){standardGeneric("tidy")})

#' Generic function \code{augment}
#' @description 
#' Generic function for an  statistical output augmented
#' @param  object  To be augmented
#' 
#' @export
#'  
setGeneric("augment",def=function(object){standardGeneric("augment")})

##############################################################
#################### ExpressionData ##########################
##############################################################

#' @importFrom  methods new
NULL

#' @title ExpressionData
#' S4 class
#' @description 
#' S4 class with the basic information 
#' @export ExpressionData
#' @exportClass ExpressionData 
ExpressionData = setClass("ExpressionData",slots = 
                            list(exprm  = "matrix",
                                 groups = "factor",
                                 type   = "character"))

#' Accesor for \code{exprm}
#' @description 
#' Accesor for \code{exprm}
#' @param object \code{ExpressionData} 
#' @export 
#' 
setMethod("exprm", signature = "ExpressionData",
          function(object) object@exprm)


#' Accesor for \code{exprm<-}
#' @description 
#' Accesor for \code{exprm<-}
#' @param object exprm slot of ExpressionData
#' @param value for exprm slot of ExpressionData
#' @export 
#' 
setMethod("exprm<-", "ExpressionData",
          function(object, value) {
            object@exprm <- value
            if (methods::validObject(object))
              return(object)
          })


#' Accesor for \code{groups}
#' @description 
#' Accesor for \code{groups} 
#' @param object \code{ExpressionData} 
#' @export 
#' 
setMethod("groups", signature = "ExpressionData",function(object) 
  object@groups)


#' Accesor for \code{groups<-}
#' @description 
#' Accesor for \code{groups}
#' @param object groups slot of ExpressionData
#' @param value for groups slot of ExpressionData
#' @export 
#' 
setMethod("groups<-", "ExpressionData",
          function(object, value) {
            object@groups <- value
            if (methods::validObject(object))
              return(object)
          })


#' Accesor for \code{type}
#' @description 
#' Accesor for \code{type}
#' @param object \code{ExpressionData} 
#' @export 
#' 
setMethod("type", signature = "ExpressionData",
          function(object) object@type)


#' Accesor for \code{type<-}
#' @description 
#' Accesor for \code{type<-}
#' @param object  \code{ExpressionData}
#' @param value for  \code{type}  slot of \code{ExpressionData}
#' @export 
#' 
setMethod("type<-", "ExpressionData",
          function(object, value) {
            object@type <- value
            if (methods::validObject(object))
              return(object)
          })

#' @importMethodsFrom methods show

#' @title show
#' Method for \code{ExpressionData}.
#' @description 
#' It \code{show}s and explains the different slots of an \code{ExpressionData}
#' @param  object \code{ExpressionData}
#' @export 
#' 
setMethod("show",
          signature = "ExpressionData",
          definition = function(object) {
            cat("An object of class ", class(object), "\n", sep = "")
            cat("The primary identifiers are \n",
                utils::head(rownames(exprm(object))),"\n")
            cat("The expression matrix is \n",
                utils::head(exprm(object),2),"\n")
            cat("The experimental factor is \n",groups(object),"\n")
            invisible(NULL)
          })

#' @title plot
#' Method for \code{ExpressionData}
#' @description 
#' \code{plot} method for \code{ExpressionData}
#' @param x \code{ExpressionData}
#' @param y Type of plot: 
#' \describe{
#' \item{1}{The means of log2-expression at x axis and 
#' the estimated standard error of the difference of means at y axis}
#' \item{2}{The medians at x axis and the interquantile
#' ranges at y axis.}
#' \item{3}{The two first principal components with different colours
#' for the two groups. The package \code{ggfortify} is required.}
#' }
#' @param ... Additional parameters
#' @export 
#' 
setMethod(f = "plot",
          signature = c(x="ExpressionData",y="numeric"),
          definition = function(x,y,...){
            if(y == 1){
              mu0 = rowMeans(x@exprm)
              tt = genefilter::rowttests(x@exprm,x@groups)
              sd0 = tt[,"dm"]/tt[,"statistic"]
              df = data.frame(mu0,sd0)
              df = df[sort(mu0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=mu0,y=sd0)) + 
                       ggplot2::geom_smooth() + 
                       ggplot2::xlab("Mean") + 
                       ggplot2::ylab("Standard deviation"))
            }
            if(y == 2){
              median0 = matrixStats::rowMedians(x@exprm)
              iqr0 = matrixStats::rowIQRs(x@exprm)
              df = data.frame(median0,iqr0)
              df = df[sort(median0,index.return = TRUE)$ix,]
              return(ggplot2::ggplot(df,ggplot2::aes(x=median0,y=iqr0)) + 
                       ggplot2::geom_smooth() + 
                       ggplot2::xlab("Median") + 
                       ggplot2::ylab("Interquantile Range"))
            }
            ## Plot of the two first principal components
            if(y == 3){
              xt = t(x@exprm)
              df = data.frame(xt,groups = x@groups)
              ggplot2::autoplot(stats::prcomp(xt),data=df,colour="groups")
            }
            ## Tukey mean-difference for means per condition
            if(y == 4){
              mdt = vector("list",nlevels(x@groups))
              for(i in seq_along(levels(x@groups)))
                mdt[[i]] = rowMeans(x@exprm[,x@groups == levels(x@groups)[i]])
              
              mdt_mean = (mdt[[1]] + mdt[[2]])/2
              mdt_diff = mdt[[1]] - mdt[[2]]
              df = data.frame(mean = mdt_mean,difference = mdt_diff)
              df = df[sort(df$mean,index.return = TRUE)$ix,]
              ggplot2::ggplot(df,ggplot2::aes(x=df$mean,y=df$difference)) + 
                ggplot2::geom_point() + ggplot2::ggtitle("Original scale")
              
              }
          })

setValidity("ExpressionData", function(object) {
  msg = NULL
  valid = TRUE
  if(ncol(exprm(object)) != length(groups(object))){
    valid = TRUE
    msg <- c(msg,
             "The number of columns of exprm is not equal to the length of 
             groups")
  }
  if (valid) TRUE else msg
  })



########################################################################
################# DifferentialExpressionInput ##########################
########################################################################


#' DifferentialExpressionInput 
#' S4 class for  differential expression analysis
#' @description 
#' S4 class with the basic information to perform an differential expression
#' analysis
#' @slot correction The correction method for multiple comparisons (as used
#' by \link[stats]{p.adjust}).
#' @slot test Statistical method used for marginal differential expression
#' @slot association Statistic measuring expression-phenotype association
#' @slot nulldistr Null distribution tested c("exact","randomization",
#' "bootstrap")
#' @export DifferentialExpressionInput
#' @exportClass DifferentialExpressionInput
#' 

DifferentialExpressionInput = setClass("DifferentialExpressionInput",
         slots = list(correction = "character",
                      test = "character",
                      association = "character",
                      nulldistr = "character"),
         contains = "ExpressionData")


#' @title correction
#' @param object \code{DifferentialExpressionInput}
#' @description 
#' Accesor for \code{correction}. \link[stats]{p.adjust} is used.
#' is used. 
#' @export 
setMethod("correction", signature = "DifferentialExpressionInput",
          function(object) object@correction)



#' @title test
#' @description
#' The statistical test used to test the differential expression for
#' expression matrix \code{exprm} and conditions \code{groups}.
#' @param object \code{DifferentialExpressionInput}
#' @export 
#' 
setMethod("test", signature = "DifferentialExpressionInput",
          function(object) object@test)

#' @title nulldistr
#' Accesor for \code{nulldistr}
#' @description 
#' For a given statistical \code{test} we can use the \code{exact}, 
#' \code{randomization} or \code{bootstrap} distribution.
#' @param object \code{DifferentialExpressionInput}
#' @export
setGeneric("nulldistr", function(object) standardGeneric("nulldistr"))

#' @title nulldistr
#' Accesor for \code{nulldistr}
#' @description 
#' For a given statistical \code{test} we can use the \code{exact}, 
#' \code{randomization} or \code{bootstrap} distribution.
#' @param object \code{DifferentialExpressionInput}
#' 
#' @export 
setMethod("nulldistr", signature = "DifferentialExpressionInput",
          function(object) object@nulldistr)

#' @title show
#' \code{show} method for S4 class \code{DifferentialExpressionInput}
#' @description 
#' \code{show} method for S4 class \code{DifferentialExpressionInput}
#' @param object \code{DifferentialExpressionInput}
#' @export 
setMethod("show",
          signature = "DifferentialExpressionInput",
          definition = function(object) {
            cat("An object of class ", class(object), "\n", sep = "")
            cat("The primary identifiers are \n",
                utils::head(rownames(exprm(object))),"\n")
            cat("The expression matrix is \n",
                utils::head(exprm(object),1),"\n")
            cat("The experimental factor is \n",groups(object),"\n")
            cat("The method to correct for multiple comparisons is \n",
                correction(object),"\n")
            cat("The test of the statistical method is \n",
                test(object))
            invisible(NULL)
          })

#' @title correction
#' Accesor for \code{correction}
#' @description 
#' Accesor for \code{correction}
#' @param object \code{correction} slot of \code{DifferentialExpressionInput}
#' @param value for \code{correction} slot of \code{DifferentialExpressionInput}
#' @export
setGeneric("correction<-",
           function(object, value) standardGeneric("correction<-"))

#' @title correction
#' Accesor for \code{correction}
#' @description 
#' Accesor for \code{correction}
#' @param object correction slot of \code{DifferentialExpressionInput}
#' @param value for correction slot of \code{DifferentialExpressionInput}
#' @export 
#'
setMethod("correction<-", "DifferentialExpressionInput",
          function(object, value) {
            object@correction <- value
            if (methods::validObject(object))
              return(object)
          })

#' @title test
#' Accessor 
#' @description 
#' Accesor for \code{test}
#' @param object test slot of \code{DifferentialExpressionInput}
#' @param value for test slot of \code{DifferentialExpressionInput}
#' @export
setGeneric("test<-",
           function(object, value) standardGeneric("test<-"))

#' @title test
#' Accessor 
#' @description 
#' Accessor for \code{test}
#' @param object test slot of DifferentialExpressionInput
#' @param value for test slot of DifferentialExpressionInput
#' @export 
setMethod("test<-", "DifferentialExpressionInput",
          function(object, value) {
            object@test <- value
            if (methods::validObject(object))
              return(object)
          })

#' @title association
#' Accesor for \code{association}
#' @description 
#' Accesor for \code{association}
#' @param object \code{association} slot of \code{DifferentialExpressionInput}
#' @param value for \code{association} slot of \code{DifferentialExpressionInput}
#' @export
setGeneric("association<-",
           function(object, value) standardGeneric("association<-"))

#' @title association
#' Accesor 
#' @description
#' Accesor for \code{association} 
#' @param object \code{association} slot of \code{DifferentialExpressionInput}
#' @param value for \code{association} slot of \code{DifferentialExpressionInput}
#'  c("exact","randomization","bootstrap")
#' @export 
setMethod("association<-", "DifferentialExpressionInput",
          function(object, value) {
            object@association <- value
            if (methods::validObject(object))
              return(object)
          })

#' @title nulldistr
#' Accesor 
#' @description 
#' Accesor for \code{nulldistr}
#' @param object nulldistr slot of \code{DifferentialExpressionInput}
#' @param value for  nulldistr slot of \code{DifferentialExpressionInput}
#' @export
setGeneric("nulldistr<-",
           function(object, value) standardGeneric("nulldistr<-"))

#' @title nulldistr
#' Accesor
#' @description 
#' Accesor for \code{nulldistr}
#' @param object nulldistr slot of \code{DifferentialExpressionInput}
#' @param value for  nulldistr slot of \code{DifferentialExpressionInput}
#'  c("exact","randomization","bootstrap")
#' @export 
setMethod("nulldistr<-", "DifferentialExpressionInput",
          function(object, value) {
            object@nulldistr <- value
            if (methods::validObject(object))
              return(object)
          })


#######################################################
################ DifferentialExpressionOutput #########
#######################################################
  
#' \code{DifferentialExpressionOutput} S4 class
#' @description 
#' S4 class with the basic statistical association of a statistical analysis
#' of differential expression analysis
#' @slot GeneData Data frame containing different gene identifiers
#' named "ENTREZID", "ENSEMBL", "PROBEID"
#' @slot GeneStat Data frame with columns \code{statistic}, \code{rawp}, 
#' \code{adjp}, \code{qval}
#' corresponding to the marginal differential expression analysis: the original 
#' statistic, the raw p-value, the adjusted p-value and the q-value 
#' @slot fdr False discovery rate
#' @slot foutput File to save results
#' @slot userGeneSet  The vector indices (rows of expression matrix) 
#' indicating the genes in which the user is interested
#' @export DifferentialExpressionOutput
#' @exportClass DifferentialExpressionOutput
#'  
DifferentialExpressionOutput = 
  setClass("DifferentialExpressionOutput",
         slots = list(GeneData    =  "data.frame",
                      GeneStat    =  "data.frame",
                      fdr         =  "numeric",
                      foutput     = "character",
                      userGeneSet = "vector"),
         contains = "DifferentialExpressionInput")

#' Accesor for \code{GeneData}
#' @description \code{GeneData}
#' Accesor for 
#' @param object \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("GeneData", signature = "DifferentialExpressionOutput",
          function(object) object@GeneData)

#' Accesor for \code{GeneData<-}
#' @description 
#' Accesor for \code{GeneData<-}
#' @param object DifferentialExpressionOutput 
#' @param value \code{GeneData} slot of DifferentialExpressionOutput 
#' @export 
#' 
setMethod("GeneData<-", signature = "DifferentialExpressionOutput",
           function(object, value) {
            object@GeneData <- value
            if (methods::validObject(object))
              return(object)
          })


#' Accesor for \code{GeneStat}
#' @description 
#' Accesor for \code{GeneStat}
#' @param object \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("GeneStat", signature = "DifferentialExpressionOutput",
          function(object) object@GeneStat)

#' Accesor for \code{GeneStat<-}
#' @description 
#' Accesor for \code{GeneStat<-}
#' @param object \code{DifferentialExpressionOutput}
#' @param value \code{GeneData} slot of \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("GeneStat<-", signature = "DifferentialExpressionOutput",
          function(object, value) {
            object@GeneStat <- value
            if (methods::validObject(object))
              return(object)
          })

#' Generic function for \code{foutput} 
#' @description 
#' Generic function for \code{foutput} 
#' @param object \code{foutput}  
#' @export
setGeneric("foutput", function(object) standardGeneric("foutput"))

#' Accesor for \code{foutput}
#' @description 
#' Accesor for \code{foutput}
#' @param object \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("foutput", signature = "DifferentialExpressionOutput",
          function(object) object@foutput)

#' Generic function \code{foutput<-}
#' @description 
#' Accesor for \code{foutput}
#' @param object foutput slot of \code{DifferentialExpressionOutput}
#' @param value for \code{foutput} slot of \code{DifferentialExpressionOutput}
#' @export
#' 
setGeneric("foutput<-",
           function(object, value) standardGeneric("foutput<-"))

#' Accesor for \code{foutput<-}
#' @description 
#' Accesor for \code{foutput<-}
#' @param object \code{DifferentialExpressionOutput}
#' @param value for \code{foutput} slot of \code{DifferentialExpressionOutput}
#' @export 
#' 
setMethod("foutput<-", "DifferentialExpressionOutput",
          function(object, value) {
            object@foutput <- value
            if (methods::validObject(object))
              return(object)
          })

#' Generic function for \code{fdr} 
#' @description 
#' Accesor for \code{fdr} 
#' @param object \code{fdr}  
#' @export
#' 
setGeneric("fdr", function(object) standardGeneric("fdr"))

#' Accesor for \code{fdr}
#' @description 
#' Accesor for \code{fdr}
#' @param object \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("fdr", signature = "DifferentialExpressionOutput",
          function(object) object@fdr)

#' Generic function \code{fdr<-}
#' @description 
#' Accesor for \code{fdr}
#' @param object fdr slot of \code{DifferentialExpressionOutput}
#' @param value for \code{fdr} slot of \code{DifferentialExpressionOutput}
#' @export
#' 
setGeneric("fdr<-",
           function(object, value) standardGeneric("fdr<-"))

#' Accesor for \code{fdr<-}
#' @description 
#' Accesor for \code{fdr<-}
#' @param object \code{DifferentialExpressionOutput}
#' @param value for \code{fdr} slot of \code{DifferentialExpressionOutput}
#' @export 
#' 
setMethod("fdr<-", "DifferentialExpressionOutput",
          function(object, value) {
            object@fdr <- value
            if (methods::validObject(object))
              return(object)})

#' Generic function for \code{userGeneSet} 
#' @description 
#' Generic function for \code{userGeneSet} 
#' @param object \code{userGeneSet}  
#' @export
setGeneric("userGeneSet", function(object) standardGeneric("userGeneSet"))

#' Accesor for \code{userGeneSet}
#' @description 
#' Accesor for \code{userGeneSet}
#' @param object \code{DifferentialExpressionOutput} 
#' @export 
#' 
setMethod("userGeneSet", signature = "DifferentialExpressionOutput",
          function(object) object@userGeneSet)

#' Generic function \code{userGeneSet<-}
#' @description 
#' Generic function for \code{userGeneSet}
#' @param object userGeneSet slot of \code{DifferentialExpressionOutput}
#' @param value for \code{userGeneSet} slot of \code{DifferentialExpressionOutput}
#' @export
#' 
setGeneric("userGeneSet<-",
           function(object, value) standardGeneric("userGeneSet<-"))

#' Accesor for \code{userGeneSet<-}
#' @description 
#' Accesor for \code{userGeneSet<-}
#' @param object \code{DifferentialExpressionOutput}
#' @param value for \code{userGeneSet} slot of \code{DifferentialExpressionOutput}
#' @export 
#' 
setMethod("userGeneSet<-", "DifferentialExpressionOutput",
          function(object, value) {
            object@userGeneSet <- value
            if (methods::validObject(object))
              return(object)
          })


#' @title tidy
#' Method for \code{DifferentialExpressionOutput}
#' @description 
#' It generates a tidy report of the differential expression analysis
#' from a (S4 object) \code{DifferentialExpressionOutput}. 
#' @param object  \code{DifferentialExpressionOutput}
#'  
#' @export 
#'  
setMethod("tidy",
          signature = "DifferentialExpressionOutput",
          definition = function(object){
            sig = which(object@GeneStat[,"adjp"]<= object@fdr)       
            data.frame(object@GeneData[sig,],object@GeneStat[sig,])
          })


#' @title glimpse
#' Method for \code{DifferentialExpressionOutput}
#' @description 
#' It generates a report in a html file with the results of 
#' the differential expression analysis
#' from a (S4 object) \code{DifferentialExpressionOutput}. 
#' @param object  \code{DifferentialExpressionOutput}
#'  
#' @export 
#'  
setMethod("glimpse",
          signature = "DifferentialExpressionOutput",
          definition = function(object){
            if(is.element("ENTREZID",names(object@GeneData))) 
              object@GeneData[,"ENTREZID"] = 
                entrezid2url(object@GeneData[,"ENTREZID"])
            if(is.element("ENSEMBL",names(object@GeneData))) 
              object@GeneData[,"ENSEMBL"] = 
                ensembl2url(object@GeneData[,"ENSEMBL"])
            if(is.element("GO",names(object@GeneData))) 
              object@GeneData[,"GO"] = go2url(object@GeneData[,"GO"])
            sig = which(object@GeneStat[,"adjp"]< object@fdr)  
            
            df0 = data.frame(object@GeneData[sig,],object@GeneStat[sig,])      
            htmlRep = ReportingTools::HTMLReport(
              shortName =object@foutput,title = object@foutput,
              reportDirectory = "./reports")
            ReportingTools::publish(df0,htmlRep)
            ReportingTools::finish(htmlRep)
          })



#' \code{augment} method for \code{DifferentialExpressionOutput}
#' @description 
#' It generates a report in a html file with the results of 
#' the differential expression analysis
#' from a (S4 object) \code{DifferentialExpressionOutput}.
#' Links to plots with a graphical display of the expression profile 
#' is provided. 
#' @param object  \code{DifferentialExpressionOutput}
#' @export 
#'  
setMethod("augment",
          signature = "DifferentialExpressionOutput",
          definition = function(object){
              if(!dir.exists("./reports/"))  dir.create("./reports/")
              if(!dir.exists("./reports/figures/"))  
                  dir.create("./reports/figures/")
              if(is.element("ENTREZID",names(object@GeneData))) 
                  object@GeneData[,"ENTREZID"] = 
                      entrezid2url(object@GeneData[,"ENTREZID"])
              if(is.element("ENSEMBL",names(object@GeneData))) 
                  object@GeneData[,"ENSEMBL"] = 
                      ensembl2url(object@GeneData[,"ENSEMBL"])
              if(is.element("GO",names(object@GeneData))) 
                  object@GeneData[,"GO"] = tami::go2url(object@GeneData[,"GO"]) 
              
              if(is.logical(object@userGeneSet)) 
                  object@userGeneSet = sort(object@GeneStat[,"adjp"],
                                            index.return=TRUE)$ix[1:100] 
            rowPlot = function(i){ 
              df = data.frame(condition=object@groups,
                              expression=object@exprm[i,]) 
              p = ggplot2::ggplot(df, ggplot2::aes(x=df$condition, 
                                                   y=df$expression,
                                                   color=df$condition))+ 
                ggplot2::geom_jitter(position=ggplot2::position_jitter(0.2))+
                ggplot2::stat_summary(fun.data="mean_sdl",geom="pointrange")+ 
                ggplot2::coord_flip() 
              return(ggplot2::ggsave(filename = paste0(object@foutput,i,".png"),
                                     plot=p,path = "./reports/figures/"))}
            devnull = sapply(object@userGeneSet,rowPlot) 
            profile = vector("character",nrow(object@GeneData)) 
            profile[object@userGeneSet] = 
              paste0(getwd(),"/reports/figures/",object@foutput,
                     object@userGeneSet,".png") 
            profile[object@userGeneSet] = 
              paste0("<a href='file://",profile[object@userGeneSet],
                     "'>","profile","</a>")
            profile[-object@userGeneSet] = NA 
            sig = which(object@GeneStat[,"adjp"]<object@fdr)
            sig = union(sig,object@userGeneSet) 
            
            df0 = data.frame(object@GeneData[sig,],object@GeneStat[sig,],
                             Profile = profile[sig]) 
            htmlRep = ReportingTools::HTMLReport(
              shortName =object@foutput,title = object@foutput,
              reportDirectory = "./reports")
            ReportingTools::publish(df0,htmlRep)
            ReportingTools::finish(htmlRep)})


################################################################
##################### GENE SET ANALYSIS ########################
################################################################

#' S4 class with the input to perform gene set analysis
#' @description 
#' S4 class with the data to perform a gene set analysis
#' @slot  GeneSetList \code{list} containing the gene sets. Each gene set is an 
#' element of the \code{list} and the \code{name} of this element is the name 
#' of the 
#' group with annotation data for the gene sets for instance, GO, KEGG, ...)
#' @slot EnrichmentMethod Enrichment method 

#' @details 
#' The different \code{method}s implemented are 
#' 1. \code{ora} Over-representation analysis with \code{SignificantGeneSet} 
#' 2. Significant gene set is defined as set0 = which(GeneStat[,"qval"] < fdr)
#' and an over-representation analysis is performed.
#' 3. An over-representation analysis using as significant gene set 
#' \code{SignificantGeneSet}
#' @export 
#'  
setClass("GSAInput",
         slots = list(GeneSetList        = "list",
                      EnrichmentMethod   = "character"),
         contains = "DifferentialExpressionOutput")

#' S4 class for output of gene set analysis
#' @description 
#' S4 class to generate outputs of gene set analysis
#' @slot  GeneSetData \code{data.frame} containing the gene set data (identifiers 
#' from GO, KEGG,...)
#' @slot GeneSetStat \code{data.frame} giving the different associations from 
#' expression profile and phenotipic variable: \code{statistic}, \code{rawp}, 
#' \code{adjp} and \code{qval}
#' @export 
#' 
GSAOutput = setClass("GSAOutput",
                     slots = list(GeneSetData        = "data.frame",
                                  GeneSetStat        = "data.frame"),
                     contains = "GSAInput")


#' Accesor for \code{GeneSetList}
#' @description \code{GeneSetList}
#' Accesor for 
#' @param object \code{GSAInput} 
#' @export 
#' 
setMethod("GeneSetList", signature = "GSAInput",
          function(object) object@GeneSetList)


#' Accesor for \code{GeneSetList<-}
#' @description 
#' Accesor for \code{GeneSetList<-}
#' @param object \code{GSAInput} 
#' @param value \code{GeneSetList} slot of \code{GSAInput} 
#' @export 
#' 
setMethod("GeneSetList<-", signature = "GSAInput",
          function(object, value) {
            object@GeneSetList <- value
            if (methods::validObject(object))
              return(object)
          })

#' Method \code{glimpse} for \code{GSAOutput}
#' @description 
#' It generates a glimpse report of the gene set analysis
#' from a (S4 object) \code{GSAOutput}. 
#' @param object  \code{GSAOutput} 
#' @export 
#' 
setMethod("glimpse",
          signature = "GSAOutput",
          definition = function(object){
            if(is.element("GO",names(object@GeneSetData))) 
              object@GeneSetData[,"GO"] = go2url(object@GeneSetData[,"GO"])
            if(is.element("KEGG",names(object@GeneSetData))) 
              object@GeneSetData[,"KEGG"] = kegg2url(object@GeneSetData[,"KEGG"])
            df0 = data.frame(object@GeneSetData,object@GeneSetStat)
             htmlRep = ReportingTools::HTMLReport(
              shortName =object@foutput,
              title = object@foutput,
              reportDirectory = "./reports")
            ReportingTools::publish(df0,htmlRep)
            ReportingTools::finish(htmlRep)
          })

#' \code{tidy} method for \code{GSAOutput}
#' @description 
#' It generates a tidy report of the gene set analysis
#' from a (S4 object) \code{GSAOutput}. 
#' @param object  \code{GSAOutput} 
#' @export 
#'  
setMethod("tidy",
          signature = "GSAOutput",
          definition = function(object){
           data.frame(object@GeneSetData,object@GeneSetStat)
          })

#-------- Conversion from ExpressionSet and RangedSummarizedExperiment to 
#-------- ExpressionData

#' Convert
#' @description 
#' Convert from \code{ExpressionSet} and \code{RangedSummarizedExperiment} to
#' \code{ExpressionData}
#' @param x \code{ExpressionSet} or \code{RangedSummarizedExperiment}
#' @param y Two-level factor or its name
#' @export
convert = function(x,y = NULL){
  xed = NULL      
  if(class(y)[1] == "character"){ 
    if(class(x)[1] == "ExpressionSet")
      xed = ExpressionData(exprm=Biobase::exprs(x),
                           groups=Biobase::pData(x)[,y],
                           type="ExpressionSet")
    if(class(x)[1] == "RangedSummarizedExperiment" |
       class(x)[1] == "SummarizedExperiment")
      xed = ExpressionData(exprm=SummarizedExperiment::assay(x),
                           groups=SummarizedExperiment::colData(x)[,y],
                           type="RangedSummarizedExperiment")
  }
  if(class(y)[1] == "factor"){ 
    if(class(x)[1] == "ExpressionSet")
      xed = ExpressionData(exprm=Biobase::exprs(x),
                           groups=y,type="ExpressionSet")
    if(class(x)[1] == "RangedSummarizedExperiment" |
       class(x)[1] == "SummarizedExperiment")
      xed = ExpressionData(exprm=SummarizedExperiment::assay(x),
                           groups=y,
                           type="RangedSummarizedExperiment")
  }
  if(class(x)[1] == "ExpressionData" & is.null(y)) xed = x 
  xed
}

